#!/usr/bin/env python3
# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest
from sfsdk import cli, containers, log, imgsettings, techsettings
import tempfile
import os
import threading
import time

base_dir = tempfile.mkdtemp(prefix='sfsdk-tests-') 

# Needed here for the unittest.skip
containers.load()

class TestWatch(unittest.TestCase):

    def setUp(self):

        containers.load()
        imgsettings.load(base_dir)
        techsettings.load(base_dir)

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_watch_container(self):
        cli.img_add(
                name = 'sdktest-watch', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        imgsettings.settings['images']['sdktest-watch']['container']['external_rdp_port'] = '19999'
        imgsettings.settings['images']['sdktest-watch']['container']['external_http_port'] = '19998'

        cli.img_build(
                name = 'sdktest-watch',
                source= 'default',
                skip_base_img = False
                )
        cli.img_run(
                name = 'sdktest-watch',
                force = True
                )

        # Can't use cli.img_watch here because it doesn't exit
        cli.containers.watch_container('sdktest-watch-inst')
        _, output = cli.containers.exec_on_container(
                'sdktest-watch-inst',  
                """sh -c "touch /home/sf/test && sleep 2 && cat /tmp/sfsdkwatch.json | jq -r '.[0]'" """)
        self.assertEqual(output, "/home/sf/test")

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_img_watch(self):
        cli.img_add(
                name = 'sdktest-watch', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        imgsettings.settings['images']['sdktest-watch']['container']['external_rdp_port'] = '19999'
        imgsettings.settings['images']['sdktest-watch']['container']['external_http_port'] = '19998'

        cli.img_build(
                name = 'sdktest-watch',
                source= 'default',
                skip_base_img = False
                )
        cli.img_run(
                name = 'sdktest-watch',
                force = True
                )

        # Prepare the container to reduce waiting time
        exit_code, output = cli.containers.exec_on_container( 
                'sdktest-watch-inst',  
                "sh -c \
                'apt-get update && \
                apt-get install -y python3-watchdog'")
        self.assertEqual(exit_code, 0)

        # Run watch
        p = threading.Thread(target=cli.img_watch, args=('sdktest-watch','/home/sf', 10))
        p.start()

        time.sleep(5)
        
        exit_code, output = cli.containers.exec_on_container(
                'sdktest-watch-inst',  
                """sh -c "touch /home/sf/test" """)

        time.sleep(10)

        with open(os.path.join(imgsettings.settings['images']['sdktest-watch']['build_dir'], '.sfsdk-watch.txt'), 'r') as stream:
            self.assertEqual(stream.read(), "/home/sf/test")

        p.join()

    def tearDown(self):

        imgsettings.remove_image('sdktest-watch')

        for container in containers.dockerc.containers.list(all=True):
            if container.name.startswith('sdktest'):
                container.remove(force=True)

        for img in containers.dockerc.images.list():
            for tag in img.tags:
                if tag.startswith('sdktest'):
                    containers.dockerc.images.remove(tag, force=True)

        containers.dockerc.close()

if __name__ == '__main__':
    unittest.main()



# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from sfsdk import log
import os
import getpass
from ruamel import yaml
from sfsdk import remote

settings = {}
settings_path = None
current_source = ''

default_hub_url = 'https://hub.secureflag.com'

def load(base_dir):
    """ Load or create authentication settings file"""
    global settings, settings_path

    settings_path = os.path.join(base_dir, 'login.yml')

    try:
        with open(settings_path, 'r') as yaml_file:
            settings = yaml.safe_load(yaml_file)
    except OSError:
        os.makedirs(base_dir, exist_ok=True)
        settings = {
                'default_source': 'default',
                'sources' : {
                    'default' : {
                        'type': '',
                        'username': '',
                        'password': ''
                    },
                }
            }
        save_to_file()
    except yaml.YAMLError:
        raise log.FatalMsg("Settings file %s has broken YAML, delete it to recreate" % settings_path)

    if not settings or not type(settings) is dict:
        raise log.FatalMsg("Settings file %s is not correct, delete it to recreate" % settings_path)

def save_to_file():
    """ Save settings file"""
    global settings

    try:
        with open(settings_path, 'w+') as outfile:
            yaml.dump(settings, outfile, default_flow_style=False)
    except OSError as e:
        log.debug(str(e))
    except yaml.YAMLError as e:
        raise log.FatalMsg("Error saving current settings" % settings_path)

def set_source(source):

    global current_source

    # Check source, default_source if empty
    if not source:
        current_source = settings.get('default_source')
    else:
        current_source = source

    if not current_source: 
        raise log.FatalMsg("Source is not set, re-run with --source or add 'default_source' to %s" % settings_path)

def load_or_ask_for_creds(doing_config = False):

    global settings

    source_settings = settings.get('sources', {}).get(current_source, {})

    if not doing_config and (
            not source_settings.get('username') or 
            not source_settings.get('password') or 
            not source_settings.get('type')
        ):
        log.warn("The authentication to the SecureFlag source '%s' hasn't been configured yet. Run 'sfsdk config' to avoid to be prompted again." % (current_source))

    source_type = source_settings.get('type')
    if doing_config or not source_type:
        log.info("Do you manage a SecureFlag Community Edition deployment?")
        source_type_ans = input('Yes or No [%s]: ' % ('Yes' if source_type == 'deployment' else 'No') )
        if source_type_ans.lower().startswith('y'):
            source_type = 'deployment'
        elif source_type_ans.lower().startswith('n'):
            source_type = 'hub'
        elif source_type_ans == '':
            source_type = 'hub'
        else:
            raise log.FatalMsg("Unexpected answer")

    if source_type == 'deployment':
        url = source_settings.get('url', '')
        if not url or doing_config:
            log.info('Please type the URL for your SecureFlag deployment')
            new_url = input('URL [%s]: ' % url)
            if not new_url or not new_url.startswith('https://'):
                raise log.FatalMsg("URL error, please type the full URL e.g. https://...")
            else:
                url = new_url
    else:
        url = source_settings.get('url', default_hub_url)

    username = source_settings.get('username', '')
    if not username or doing_config:
        if source_type == 'deployment':
            log.info('Please type the username for your SFAdmin user')
        else:
            log.info('Please type the username for your SecureFlag Exercise Hub account')

        new_username = input('Username: ')
        if new_username:
            username = new_username
        else:
            raise log.FatalMsg("Empty username")

    password = source_settings.get('password', '')
    if not password or doing_config:
        log.info('Please type %s\'s password' % (username))
        new_password = getpass.getpass('Password: ')
        if new_password:
            password = new_password
        else:
            raise log.FatalMsg("Empty password")

    # Merge last inserted with the loaded config
    settings['sources'][current_source].update({
        'type': source_type,
        'username': username,
        'password': password,
        'url': url
    })

    return source_type, url, username, password



#!/usr/bin/env python2

# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import json
import time
import os
import sys
from threading import Lock
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

modified_files = []
lock = Lock()
folder_to_monitor = None

def on_any_event(event):

    global folder_to_monitor

    path_to_add = event.src_path
    if path_to_add == folder_to_monitor:
        return

    # Cut the path to have just the piece just after the / 
    # of the folder to monitor
    path_to_add = os.path.sep.join(
        path_to_add.split(os.path.sep)[0:folder_to_monitor.count('/')+2]
    )

    # Add the trailing slash if it's a folder
    if os.path.isdir(path_to_add):
        path_to_add = os.path.join(path_to_add, '')

    lock.acquire()
    if not path_to_add in modified_files:


        modified_files.append(path_to_add)

        with open(status_json, 'w') as outfile:
            json.dump(modified_files, outfile, indent=2)

    lock.release()

if __name__ == "__main__":

    if len(sys.argv) != 3:
        print('Usage: sfsdkwatch.py <folder> <output file>')
        sys.exit(1)

    folder_to_monitor = sys.argv[1].rstrip('/') # No ending slash
    status_json = sys.argv[2]

    event_handler = PatternMatchingEventHandler(
            ignore_patterns=[
                "/home/sf/.Xauthority*",
                "/home/sf/.xorgxrdp.*",
                "/home/sf/.xsession-errors",
                "/home/sf/.dbus/**",
                "/home/sf/.dbus",
                "/home/sf/.cache/**",
                "/home/sf/.cache",
                "/home/sf/thinclient_drives/**",
                "/home/sf/thinclient_drives",
                "/home/sf/.ICEauthority*",
                "/home/sf/.gvfs/**",
                "/home/sf/.gvfs",
                "/home/sf/exercise/app/**",
                "/home/sf/exercise/app",
                "/home/sf/exercise/.sf/**",
                "/home/sf/exercise/.sf",
                ],
            ignore_directories=False,
            case_sensitive=True
            )
    event_handler.on_any_event = on_any_event

    observer = Observer()
    observer.schedule(event_handler, folder_to_monitor, recursive=True)
    observer.start()

    # Create an empty json when ready
    with open(status_json, 'w') as outfile:
        json.dump([], outfile)

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
                observer.stop()
    observer.join()



# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import re
import os
import glob
import zipfile
import json
import shutil
import base64
import copy
import uuid
from sfsdk import log, utils
from ruamel import yaml
from pathlib import Path

workspace_dir = None

def load(base_dir):

    global workspace_dir

    workspace_dir = os.path.join(
            base_dir,
            'exr'
    )

    if not os.path.exists(workspace_dir):
        os.makedirs(workspace_dir)

def get_standard_name(exercise_data):

     tech = re.sub(
            "[^a-z0-9]+",
            "_",
            exercise_data['technology'].lower()
            )
            
     framework = re.sub(
            "[^a-z0-9]+",
            "_",
            # TODO: make sure all the names has a framework
            str(exercise_data.get('framework')).lower()
            )

     title = re.sub(
            "[^a-z0-9]+",
            "_",
            exercise_data['title'].lower()
            )

     long_uuid = exercise_data.get('uuid', str(uuid.uuid4()))

     if long_uuid[:6] in ('local-', 'sfsdk-'):
         shorter_uuid = long_uuid[:10]
     else:
         shorter_uuid = long_uuid[:4]

     return '%s_%s_%s-%s' % (
         tech,
         framework, 
         title, 
         shorter_uuid
        )


def get_standard_flag_name(flag_data):

     title = re.sub(
            "[^a-z0-9]+",
            "_",
            flag_data['title'].lower()
            )

     flag_type = flag_data['flagList'][0]['type'].lower()

     return '%s_%s' % (title, flag_type)


def ls_exercises():

    exercises = []
    for exercise_yml_path in glob.glob('%s/*/exercise.yml' % workspace_dir):
        exercises.append(exercise_yml_path.split('/')[-2])

    return exercises

def get_exercises(raw = False):

    exercises = {}

    for exercise_yml_path in glob.glob('%s/*/exercise.yml' % workspace_dir):
        exercise_name = exercise_yml_path.split('/')[-2]
        exercise_meta = load_exercise_meta(exercise_name)

        if not raw:

            exercises[exercise_name] = {}

            for field in (
                    'id', 'uuid', 'fromHub', 'title', 'subtitle', 'author', 'score', 
                    'technology', 'trophyname', 'duration', 'difficulty', 'description', 
                    'lastUpdate', 'exerciseType', 'status', 'tags'):

                exercises[exercise_name][field] = exercise_meta.get(field)

        else:

             exercises[exercise_name] = exercise_meta


    return exercises

def get_exercise_by_uuid(uuid):

    for exercise_yml_path in glob.glob('%s/*/exercise.yml' % workspace_dir):

        exercise_name = exercise_yml_path.split('/')[-2]
        exercise_meta = load_exercise_meta(exercise_name)

        if exercise_meta['uuid'] == uuid:
            return exercise_meta

    return {}

def get_exercise_by_name(name):

    exercise_name = None
    for exercise_yml_path in glob.glob('%s/*/exercise.yml' % workspace_dir):

        current_name = exercise_yml_path.split('/')[-2]

        if current_name == name:
            exercise_name = current_name
            break

    if exercise_name:
        return load_exercise_meta(exercise_name)
    else:
        raise log.FatalMsg('The exercise does not exist locally')

def remove_exercise(exercise):

    exercise_dir = os.path.join(workspace_dir, exercise)

    if not os.path.exists(exercise_dir):
        raise log.FatalMsg("Can't find exercise directory")

    if not Path(workspace_dir) in Path(exercise_dir).parents:
        raise log.FatalMsg("Wrong folder")

    shutil.rmtree(exercise_dir)


def load_exercise_meta(exercise):

    exercise_dir = os.path.join(workspace_dir, exercise)

    if not os.path.exists(exercise_dir):
        os.makedirs(exercise_dir)

    exercise_meta_path = os.path.join(exercise_dir, 'exercise.yml')

    exercise_yaml = {}

    try:

        with open(exercise_meta_path, 'r') as stream:

            try:
                exercise_yaml = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                raise log.FatalMsg(exc)

            with open(os.path.join(exercise_dir, 'description.txt.md'), 'r') as stream:
                exercise_yaml['description'] = utils.load_and_replace_media_with_b64(
                    exercise_dir,
                    stream.read()
                )

            with open(os.path.join(exercise_dir, 'information.md'), 'r') as stream:
                exercise_yaml['information']['text'] = utils.load_and_replace_media_with_b64(
                    exercise_dir,
                    stream.read()
                )

            with open(os.path.join(exercise_dir, 'solution.md'), 'r') as stream:
                exercise_yaml['solution']['text'] = utils.load_and_replace_media_with_b64(
                    exercise_dir,
                    stream.read()
                )

            # Skip stack KB
            #with open(os.path.join(exercise_dir, 'stack.md'), 'r') as stream:
            #    exercise_yaml['stack']['md']['text'] = stream.read()

            # Dirty validation on multiple flags with the same name
            added_flag_names = []

            for i, flag_yaml in enumerate(exercise_yaml['flags']):

                flag_name = get_standard_flag_name(flag_yaml)

                if flag_name in added_flag_names:
                    raise log.FatalMsg("Error, flag %s with the same name detected" % (flag_name))
                added_flag_names.append(flag_name)

                # Skip vuln KB
                #flag_path = 'flag_%s_kb.md' % flag_name

                #with open(os.path.join(exercise_dir, flag_path), 'r') as stream:
                #    exercise_yaml['flags'][i]['kb']['md']['text'] = stream.read()

                # Dirty validation here
                if len(flag_yaml['flagList']) > 1:
                    raise log.FatalMsg("Error, more than one flagList per flag")

                flaglist = flag_yaml['flagList'][0]

                flaglist_md_path = 'flag_%s_text.md' % (flag_name)

                # Check if the old flag was named in the same way
                if (
                    exercise_yaml['flags'][i]['flagList'][0]['md']['text'] !=
                    f"!import {flaglist_md_path}"
                ):
                    raise log.FatalMsg(
                        f"Mismatch between the expected flag file '{flaglist_md_path}' and the previous import '{exercise_yaml['flags'][i]['flagList'][0]['md']['text']}'"
                        )

                with open(os.path.join(exercise_dir, flaglist_md_path), 'r') as stream:
                    exercise_yaml['flags'][i]['flagList'][0]['md']['text'] = utils.load_and_replace_media_with_b64(
                        exercise_dir,
                        stream.read()
                    )

                if 'hint' in flaglist and 'text' in flaglist['hint']['md']:
                    flaglist_hint_md_path = 'flag_%s_hint.md' % (flag_name)

                    # Check if the old flag was named in the same way
                    if (
                        exercise_yaml['flags'][i]['flagList'][0]['hint']['md']['text'] !=
                        f"!import {flaglist_hint_md_path}"
                    ):
                        raise log.FatalMsg(
                            f"Mismatch between the expected flag file '{flaglist_hint_md_path}' and the previous import '{exercise_yaml['flags'][i]['flagList'][0]['hint']['md']['text']}'"
                            )

                    with open(os.path.join(exercise_dir, flaglist_hint_md_path), 'r') as stream:
                        exercise_yaml['flags'][i]['flagList'][0]['hint']['md']['text'] = utils.load_and_replace_media_with_b64(
                            exercise_dir,
                            stream.read()
                        )

    except FileNotFoundError as e:
        raise log.FatalMsg(f"Can't load the metadata: {e.filename} was expected but not found.")

    return exercise_yaml

def save_exercise_meta(exercise_yaml_orig):

    exercise_yaml = copy.deepcopy(exercise_yaml_orig)

    exercise = get_standard_name(exercise_yaml)

    exercise_dir = os.path.join(workspace_dir, exercise)

    if not os.path.exists(exercise_dir):
        os.makedirs(exercise_dir)

    if log.verbose:
        exercise_meta_debug_path = os.path.join(exercise_dir, '.last-saved.json')
        log.debug('Saving JSON debug to %s' % (exercise_meta_debug_path))
        with open(exercise_meta_debug_path, 'w') as f:
              json.dump(exercise_yaml, f, indent=4)

    description = exercise_yaml['description']
    description = utils.save_and_replace_b64_with_media(
        base_folder = exercise_dir,
        content = description,
        content_name = 'description'
    )
    with open(os.path.join(exercise_dir, 'description.txt.md'), 'w+') as stream:
        stream.write(description)

    exercise_yaml['description'] = '!import description.txt.md'

    information_text = exercise_yaml['information']['text']
    information_text = utils.save_and_replace_b64_with_media(
        base_folder = exercise_dir,
        content = information_text,
        content_name = 'information'
    )
    with open(os.path.join(exercise_dir, 'information.md'), 'w+') as stream:
        stream.write(information_text)

    exercise_yaml['information']['text'] = '!import information.md'

    solution_text = exercise_yaml['solution']['text']
    solution_text = utils.save_and_replace_b64_with_media(
        base_folder = exercise_dir,
        content = solution_text,
        content_name = 'information'
    )
    with open(os.path.join(exercise_dir, 'solution.md'), 'w+') as stream:
        stream.write(solution_text)

    exercise_yaml['solution']['text'] = '!import solution.md'

    # Do not store the complete stack KB, but only variant and technology
    try:
        exercise_yaml['stack'] = {
            'technology' : exercise_yaml['stack']['technology'],
            'variant': exercise_yaml['stack']['variant'],
            'uuid': exercise_yaml['stack']['uuid']
        }
    except KeyError as e:
        raise log.FatalMsg(f"Metadata parsing error {e}")


    # Dirty validation on multiple flags with the same name
    added_flag_names = []

    for i, flag_yaml in enumerate(exercise_yaml['flags']):

        flag_name = get_standard_flag_name(flag_yaml)

        if flag_name in added_flag_names:
            raise log.FatalMsg("Error, flag %s with the same name detected" % (flag_name))
        added_flag_names.append(flag_name)

        try:
            # Do not store the complete vulnerability KB, but only category, vulnerability, technology, and isAgnostic
            exercise_yaml['flags'][i]['kb'] = {
                'category' : exercise_yaml['flags'][i]['kb']['category'],
                'vulnerability': exercise_yaml['flags'][i]['kb']['vulnerability'],
                'technology': exercise_yaml['flags'][i]['kb']['technology'],
                'isAgnostic': exercise_yaml['flags'][i]['kb']['isAgnostic'],
                'uuid': exercise_yaml['flags'][i]['kb']['uuid']
            }
        except KeyError as e:
            raise log.FatalMsg(f"Metadata parsing error {e}")

        # Dirty validation on multiple flagList
        if len(flag_yaml['flagList']) > 1:
            raise log.FatalMsg("Error, more than one flagList per flag")

        flaglist = flag_yaml['flagList'][0]
            
        flaglist_md = flaglist['md']['text']
        flaglist_md_path = 'flag_%s_text.md' % (flag_name)

        flaglist_md = utils.save_and_replace_b64_with_media(
            base_folder = exercise_dir,
            content = flaglist_md,
            content_name = flag_name
        )

        with open(os.path.join(exercise_dir, flaglist_md_path), 'w+') as stream:
            stream.write(flaglist_md)

        exercise_yaml['flags'][i]['flagList'][0]['md']['text'] = '!import %s' % flaglist_md_path

        if 'hint' in flaglist and 'text' in flaglist['hint']['md']:
            flaglist_hint_md = flaglist['hint']['md']['text']
            flaglist_hint_md_path = 'flag_%s_hint.md' % (flag_name)

            flaglist_hint_md = utils.save_and_replace_b64_with_media(
                base_folder = exercise_dir,
                content = flaglist_hint_md,
                content_name = f'{flag_name}_hint'
            )

            with open(os.path.join(exercise_dir, flaglist_hint_md_path), 'w+') as stream:
                stream.write(flaglist_hint_md)

            exercise_yaml['flags'][i]['flagList'][0]['hint']['md']['text'] = '!import %s' % flaglist_hint_md_path
    
    fix_incoming_exercise_data(exercise_yaml)

    with open(os.path.join(exercise_dir, "exercise.yml"), "w") as f:
        yaml.dump(exercise_yaml, f, default_flow_style=False, width=float("inf"))

    return exercise_dir


def update_exercise_folder(old_exercise_dir, new_exercise_data):

    new_exercise_dir = save_exercise_meta(new_exercise_data)

    if old_exercise_dir != new_exercise_dir:
        log.warn("Moved from %s to %s" % (utils.prettypath(old_exercise_dir), utils.prettypath(new_exercise_dir)))
        shutil.rmtree(old_exercise_dir)


def get_exercise_environment_variable(exercise_name):

    exercise_settings = get_exercise_by_name(exercise_name)
    return base64.b64encode(json.dumps(
            {
                'title': exercise_settings['title'],
                'flags': exercise_settings['flags']
            }
        ).encode()
    )

def generate_exercise_environment(flag_name):

    return base64.b64encode(json.dumps(
            {
                'flags': [{"flagList":[{"selfCheck":{"name": flag_name }}]}]
            }
        ).encode()
    )

def fix_incoming_exercise_data(exercise_json):

    # Int-ify status
    exercise_json['status'] = int(exercise_json['status'])

    # Sort by flag type, to keep always the same order
    exercise_json['flags'].sort(key=lambda f: f['flagList'][0]['type'])

    # Delete ids
    utils.recursive_del(exercise_json['flags'], 'id')
    
    try:
        del exercise_json['information']['id']
        del exercise_json['solution']['id']
    except KeyError:
        pass

def fix_outgoing_exercise_data(exercise_json):

    # Int-ify status
    exercise_json['status'] = int(exercise_json['status'])

    # Delete any reference to KBs and Stack UUIDs, let the platform guess by other meta
    utils.recursive_del(exercise_json['flags'], 'uuid')
    utils.recursive_del(exercise_json['stack'], 'uuid')

    # Delete md.id in flags that cause some issue
    utils.recursive_del(exercise_json['flags'], 'id')

def get_path(exercise_name):
   return os.path.join(workspace_dir, exercise_name)

def get_pretty_path(exercise_name):
   return utils.prettypath(os.path.join(workspace_dir, exercise_name))


# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from sfsdk import exrsettings, log, utils
from ruamel import yaml
import os
import glob
import re
import shutil
import json
import copy
import uuid
import tempfile

workspace_dir = None

def load(base_dir):

    global workspace_dir

    workspace_dir = os.path.join(
            base_dir,
            'stack'
    )

    if not os.path.exists(workspace_dir):
        os.makedirs(workspace_dir)

def get_standard_stack_name(stack):

     tech = re.sub(
            "[^a-z0-9]",
            "_",
            stack['technology'].lower()
            )

     title = re.sub(
            "[^a-z0-9]",
            "_",
            stack.get('variant', '').lower()
            )

     long_uuid = stack['uuid']

     if long_uuid[:6] in ('local-', 'sfsdk-'):
         shorter_uuid = long_uuid[:10]
     else:
         shorter_uuid = long_uuid[:4]

     return '%s_%s-%s' % (tech, title, shorter_uuid)

def save_stack_meta(stack_yaml_orig):

    stack_yaml = copy.deepcopy(stack_yaml_orig)

    stack = get_standard_stack_name(stack_yaml)

    stack_dir = os.path.join(workspace_dir, stack)

    if not os.path.exists(stack_dir):
        os.makedirs(stack_dir)

    information_text = stack_yaml['md']['text']
    information_text = utils.save_and_replace_b64_with_media(
        base_folder = stack_dir,
        content = information_text,
        content_name = 'stack'
    )
    with open(os.path.join(stack_dir, 'stack.md'), 'w+') as f:
        f.write(information_text)
    stack_yaml['md']['text'] = '!import stack.md'

    if log.verbose:
        stack_meta_debug_path = os.path.join(stack_dir, '.last-saved.json')
        log.debug('Saving JSON debug to %s' % (stack_meta_debug_path))
        with open(stack_meta_debug_path, 'w') as f:
              json.dump(stack_yaml_orig, f, indent=4)

    with open(os.path.join(stack_dir, "stack.yml"), "w") as f:
        yaml.dump(stack_yaml, f, default_flow_style=False)

    return stack_dir

def load_stack_meta(stack):

    stack_dir = os.path.join(workspace_dir, stack)

    if not os.path.exists(stack_dir):
        os.makedirs(stack_dir)

    stack_meta_path = os.path.join(stack_dir, 'stack.yml')

    with open(stack_meta_path, 'r') as stream:
        try:
            stack_yaml = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            raise log.FatalMsg(exc)

        with open(os.path.join(stack_dir, 'stack.md'), 'r') as stream:
            stack_yaml['md']['text'] = utils.load_and_replace_media_with_b64(
                stack_dir,
                stream.read()
            )

    return stack_yaml

def get_stacks():

    stacks = {}

    for stack_yml_path in glob.glob('%s/*/stack.yml' % workspace_dir):

        current_name = stack_yml_path.split('/')[-2]
        
        stacks[current_name] = load_stack_meta(current_name)

    return stacks

def get_stack_by_uuid(stack_uuid):
    
    for stack_yml_path in glob.glob('%s/*/stack.yml' % workspace_dir):

        current_name = stack_yml_path.split('/')[-2]

        stack_data = load_stack_meta(current_name)

        if stack_data['uuid'] == stack_uuid:
            return stack_data

def update_stack_folder(old_stack_dir, new_stack_data):

    new_stack_dir = save_stack_meta(new_stack_data)

    if old_stack_dir != new_stack_dir:
        log.warn("Moved from %s to %s" % (utils.prettypath(old_stack_dir), utils.prettypath(new_stack_dir)))
        shutil.rmtree(old_stack_dir)

def delete_stack_by_uuid(stack_uuid):
        
    for stack_yml_path in glob.glob('%s/*/stack.yml' % workspace_dir):

        current_name = stack_yml_path.split('/')[-2]
        current_folder = os.path.dirname(stack_yml_path)

        stack_data = load_stack_meta(current_name)

        if stack_data['uuid'] == stack_uuid:

            tempfolder = tempfile.mkdtemp()

            print("Moving %s to %s" % (current_folder, tempfolder))

            os.rename(current_folder, tempfolder)

            return stack_uuid


#!/usr/bin/env python3
# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest
from sfsdk import cli, containers, log
from sfsdk import imgsettings, techsettings, authsettings, exrsettings
import threading
import time
import tempfile
import os

base_dir = tempfile.mkdtemp(prefix='sfsdk-tests-')

# Needed here for the unittest.skip
containers.load()

class TestSnapshot(unittest.TestCase):

    def setUp(self):
        containers.load()
        imgsettings.load(base_dir)
        techsettings.load(base_dir)

    def test_non_existent(self):

        self.assertRaises(log.FatalMsg, cli.img_snapshot, 
                name = 'non-existent', 
                new_name = 'non-existent-snapshot',
                force = True,
            )

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_snapshot_with_paths(self):

        cli.img_add(
                name = 'sdktestsnapshot', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        imgsettings.settings['images']['sdktestsnapshot']['container']['external_rdp_port'] = '19999'
        imgsettings.settings['images']['sdktestsnapshot']['container']['external_http_port'] = '19998'

        cli.img_build(
                name = 'sdktestsnapshot',
                source= 'default',
                skip_base_img = False
                )
        cli.img_run(
                name = 'sdktestsnapshot',
                force = True
                )

        _, _ = cli.containers.exec_on_container(
                'sdktestsnapshot-inst',  
                """sh -c "mkdir -p /home/sf/exercise/app/ /home/sf/exercise/.metadata/ && touch /home/sf/test /home/sf/exercise/app/test1 /home/sf/exercise/.metadata/test2" """)

        with open(os.path.join(imgsettings.settings['images']['sdktestsnapshot']['build_dir'], '.sfsdk-watch.txt'), 'w+') as stream:
                stream.write("/home/sf/test\n/home/sf/exercise/.metadata/test2")

        cli.img_snapshot(
                name = 'sdktestsnapshot',
                new_name = 'sdktest4',
                force = True,
                )
        cli.img_build(
                name = 'sdktest4',
                source= 'default',
                skip_base_img = False
                )
        cli.img_stop(
                names = [ 'sdktestsnapshot' ],
                )
        cli.img_rm(
                names = [ 'sdktestsnapshot' ],
                force = True
                )

        with open(os.path.join(imgsettings.settings['images']['sdktest4']['build_dir'], 'fs/home/sf/test'), 'r') as stream:
            self.assertEqual(stream.read(), "")

        with open(os.path.join(imgsettings.settings['images']['sdktest4']['build_dir'], 'app/test1'), 'r') as stream:
            self.assertEqual(stream.read(), "")

        with open(os.path.join(imgsettings.settings['images']['sdktest4']['build_dir'], 'fs/home/sf/exercise/.metadata/test2'), 'r') as stream:
            self.assertEqual(stream.read(), "")

        cli.img_build(
                name = 'sdktest4',
                source= 'default',
                skip_base_img = False
                )

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_snapshot_without_newname(self):

        # TODO: removing this shouldn't be necessary
        cli.img_rm(
                names = [ 'sdktestsnapshot' ],
                force = True
                )

        cli.img_add(
                name = 'sdktestsnapshot', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        imgsettings.settings['images']['sdktestsnapshot']['container']['external_rdp_port'] = '19999'
        imgsettings.settings['images']['sdktestsnapshot']['container']['external_http_port'] = '19998'

        cli.img_build(
                name = 'sdktestsnapshot',
                source= 'default',
                skip_base_img = False
                )
        cli.img_run(
                name = 'sdktestsnapshot',
                force = True
                )
        cli.img_snapshot(
                name = 'sdktestsnapshot',
                new_name = None,
                force = True,
                )

        imgsettings.load(base_dir)

        self.assertTrue(imgsettings.settings['images']['sdktestsnapshot-v1'])

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_snapshot_with_versioned_newname(self):
        cli.img_add(
                name = 'sdktestsnapshot', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        imgsettings.settings['images']['sdktestsnapshot']['container']['external_rdp_port'] = '19999'
        imgsettings.settings['images']['sdktestsnapshot']['container']['external_http_port'] = '19998'

        cli.img_build(
                name = 'sdktestsnapshot',
                source= 'default',
                skip_base_img = False
                )
        cli.img_run(
                name = 'sdktestsnapshot',
                force = True
                )
        cli.img_snapshot(
                name = 'sdktestsnapshot',
                new_name = None,
                force = True,
                )

        self.assertTrue(imgsettings.settings['images']['sdktestsnapshot-v1'])

        cli.img_snapshot(
                name = 'sdktestsnapshot',
                new_name = None,
                force = True,
                )

        self.assertTrue(imgsettings.settings['images']['sdktestsnapshot-v2'])

        cli.img_snapshot(
                name = 'sdktestsnapshot',
                new_name = None,
                force = True,
                )

        self.assertTrue(imgsettings.settings['images']['sdktestsnapshot-v3'])

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_snapshot(self):

        cli.img_add(
                name = 'sdktestsnapshot', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        imgsettings.settings['images']['sdktestsnapshot']['container']['external_rdp_port'] = '19999'
        imgsettings.settings['images']['sdktestsnapshot']['container']['external_http_port'] = '19998'

        cli.img_build(
                name = 'sdktestsnapshot',
                source= 'default',
                skip_base_img = False
                )
        cli.img_run(
                name = 'sdktestsnapshot',
                force = True
                )
        cli.img_snapshot(
                name = 'sdktestsnapshot',
                new_name = 'sdktest3',
                force = True,
                )
        cli.img_build(
                name = 'sdktest3',
                source= 'default',
                skip_base_img = False
                )
        cli.img_stop(
                names = [ 'sdktestsnapshot' ],
                )
        cli.img_rm(
                names = [ 'sdktestsnapshot' ],
                force = True
                )



    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_img_watch_and_snapshot(self):
        cli.img_add(
                name = 'sdktest-watch', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        imgsettings.settings['images']['sdktest-watch']['container']['external_rdp_port'] = '19999'
        imgsettings.settings['images']['sdktest-watch']['container']['external_http_port'] = '19998'

        cli.img_build(
                name = 'sdktest-watch',
                source= 'default',
                skip_base_img = False
                )
        cli.img_run(
                name = 'sdktest-watch',
                force = True
                )

        # Prepare the container to reduce waiting time
        exit_code, output = cli.containers.exec_on_container( 
                'sdktest-watch-inst',  
                "sh -c \
                'apt-get update && \
                apt-get install -y python3-watchdog'")
        self.assertEqual(exit_code, 0)

        # Run watch

        p = threading.Thread(target=cli.img_watch, args=('sdktest-watch','/home/sf', 10))
        p.start()

        time.sleep(5)
        
        exit_code, output = cli.containers.exec_on_container(
                'sdktest-watch-inst',  
                """sh -c "mkdir -p /home/sf/exercise/app/ && touch /home/sf/test /home/sf/exercise/app/test1" """)

        time.sleep(5)
        
        with open(os.path.join(imgsettings.settings['images']['sdktest-watch']['build_dir'], '.sfsdk-watch.txt'), 'r') as stream:
            self.assertEqual(
                    stream.read(), 
                    "/home/sf/test"
                    )

        cli.img_snapshot(
                name = 'sdktest-watch',
                new_name = 'sdktest-watch-snapshot',
                force = True,
                )

        with open(os.path.join(imgsettings.settings['images']['sdktest-watch-snapshot']['build_dir'], 'fs/home/sf/test'), 'r') as stream:
            self.assertEqual(stream.read(), "")

        with open(os.path.join(imgsettings.settings['images']['sdktest-watch-snapshot']['build_dir'], 'app/test1'), 'r') as stream:
            self.assertEqual(stream.read(), "")

        cli.img_build(
                name = 'sdktest-watch-snapshot',
                source= 'default',
                skip_base_img = False
                )

        p.join()

    def tearDown(self):
        for container in containers.dockerc.containers.list(all=True):
            if container.name.startswith('sdktest'):
                container.remove(force=True)

        for img in containers.dockerc.images.list():
            for tag in img.tags:
                if tag.startswith('sdktest'):
                    containers.dockerc.images.remove(tag, force=True)
        
        containers.dockerc.close()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest
from sfsdk import cli, containers, log
from sfsdk import imgsettings, techsettings, authsettings
import tempfile
import os

base_dir = tempfile.mkdtemp(prefix='sfsdk-tests-') 
settings_path = os.path.join(
        base_dir,
        'login.yml' 
    )

# Load authsettings from the real folder
auth_base_dir = os.path.expanduser('~/sf')
authsettings.load(auth_base_dir)
authsettings.set_source('developer-test-account')

class TestLogin(unittest.TestCase):

    def setUp(self):
        techsettings.load(base_dir)

    def test_empty(self):

        with open(settings_path, 'w+') as f:
            f.truncate()
        self.assertRaises(log.FatalMsg, authsettings.load, base_dir)

    def test_empty_json(self):

        with open(settings_path, 'w+') as f:
            f.write('{}')

        self.assertRaises(log.FatalMsg, authsettings.load, base_dir)

    @unittest.skipUnless(authsettings.settings.get('sources', {}).get('developer-test-account'),
                                 "No developer-test-account creds in login.yml")
    def test_developer_login(self):
        cli.img_techs('developer-test-account')

    @unittest.skipUnless(authsettings.settings.get('sources', {}).get('deployment-test-account'),
                                 "No deployment creds in login.yml")
    def test_admin_login(self):
        cli.img_techs('deployment-test-account')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest
from sfsdk import imgsettings, techsettings
from sfsdk import cli, containers, log
import tempfile
import os

base_dir = tempfile.mkdtemp(prefix='sfsdk-tests-')

# Needed here for the unittest.skip
containers.load()

class TestAdd(unittest.TestCase):

    def setUp(self):
        containers.load()
        imgsettings.load(base_dir)
        techsettings.load(base_dir)

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_duplicate(self):
        cli.img_add(
                name = 'sdktestadd', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        
        self.assertRaises(log.FatalMsg, cli.img_add, 
                name = 'sdktestadd', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None
            )

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_app_dir(self):

        app_dir = os.path.join(base_dir, 'test-app-dir')
        app_file = os.path.join(app_dir, 'file.java')
        os.makedirs(app_dir)

        with open(app_file, 'w+') as stream:
            stream.write('1')
        
        cli.img_add(
                name = 'sdktestadd-app-dir', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = app_dir)

        with open(os.path.join(imgsettings.settings['images']['sdktestadd-app-dir']['build_dir'], 'app', 'file.java'), 'r') as stream:
            self.assertEqual(stream.read(), "1")

    def tearDown(self):

        for container in containers.dockerc.containers.list(all=True):
            if container.name.startswith('sdktest'):
                container.remove(force=True)

        for img in containers.dockerc.images.list():
            for tag in img.tags:
                if tag.startswith('sdktest'):
                    containers.dockerc.images.remove(tag, force=True)

        containers.dockerc.close()

if __name__ == '__main__':
    unittest.main()



# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from sfsdk import log, utils
from ruamel import yaml
import os
import glob
import re
import shutil
import json
import copy

workspace_dir = None

def load(base_dir):
    
    global workspace_dir

    workspace_dir = base_dir

def save_framework(framework):

    framework_path = os.path.join(workspace_dir, 'frameworks.yml')

    try:
        with open(framework_path, "r") as f:
            frameworks_list = yaml.safe_load(f)
    except OSError:
        frameworks_list = []
    except yaml.YAMLError:
        raise log.FatalMsg("Settings file %s has broken YAML, delete it to recreate" % framework_path)

    if not frameworks_list or not isinstance(frameworks_list, list):
        frameworks_list = []

    # Add if it does not exist
    if not framework in [ n['name'] for n in frameworks_list ]:

        ids = [ n['id'] for n in frameworks_list ]

        # Pick the first available id from 0 to 1000
        random_id = next(iter(set(range(0, 1000)) - set(ids)))

        frameworks_list.append({
            'id': random_id,
            'name': framework,
            'local': True
        })

    with open(framework_path, "w+") as f:
        yaml.dump(frameworks_list, f, default_flow_style=False)

    return framework_path

def get_frameworks():

    framework_path = os.path.join(workspace_dir, 'frameworks.yml')

    try:
        with open(framework_path, "r") as f:
            yaml_data = yaml.safe_load(f)
    except IOError:
        yaml_data = []

    if yaml_data and isinstance(yaml_data, list):
        return yaml_data

    return []

def delete_framework_by_name(framework):
    
    framework_path = os.path.join(workspace_dir, 'frameworks.yml')

    new_framework_list = []

    with open(framework_path, "r") as f:
        frameworks_list = yaml.safe_load(f)

        if not frameworks_list or not isinstance(frameworks_list, list):
            return []

        for f in frameworks_list:
            if f['name'] != framework:
                new_framework_list.append(f)

    with open(framework_path, "w+") as f:
        yaml.dump(new_framework_list, f, default_flow_style=False)

    return new_framework_list


# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from sfsdk import exrsettings, log, utils
from ruamel import yaml
import os
import glob
import re
import shutil
import json
import copy
import uuid
import tempfile

workspace_dir = None

def load(base_dir):

    global workspace_dir

    workspace_dir = os.path.join(
            base_dir,
            'kb'
    )

    if not os.path.exists(workspace_dir):
        os.makedirs(workspace_dir)

def get_standard_kb_name(kb):

    tech = re.sub(
            "[^a-z0-9]",
            "_",
            kb['technology'].lower()
            )

    title = re.sub(
            "[^a-z0-9]",
            "_",
            kb['vulnerability'].lower()
            )

    long_uuid = kb['uuid']

    if long_uuid[:6] in ('local-', 'sfsdk-'):
         shorter_uuid = long_uuid[:10]
    else:
         shorter_uuid = long_uuid[:4]

    return '%s_%s-%s' % (tech, title, shorter_uuid)

def save_kb_meta(kb_yaml_orig):

    kb_yaml = copy.deepcopy(kb_yaml_orig)

    kb = get_standard_kb_name(kb_yaml)

    kb_dir = os.path.join(workspace_dir, kb)

    if not os.path.exists(kb_dir):
        os.makedirs(kb_dir)


    information_text = kb_yaml['md']['text']
    information_text = utils.save_and_replace_b64_with_media(
        base_folder = kb_dir,
        content = information_text,
        content_name = 'kb'
    )
    with open(os.path.join(kb_dir, 'kb.md'), 'w+') as f:
        f.write(information_text)
    kb_yaml['md']['text'] = '!import kb.md'

    if log.verbose:
        kb_meta_debug_path = os.path.join(kb_dir, '.last-saved.json')
        log.debug('Saving JSON debug to %s' % (kb_meta_debug_path))
        with open(kb_meta_debug_path, 'w') as f:
              json.dump(kb_yaml_orig, f, indent=4)

    with open(os.path.join(kb_dir, "kb.yml"), "w") as f:
        yaml.dump(kb_yaml, f, default_flow_style=False)

    return kb_dir

def load_kb_meta(kb):

    kb_dir = os.path.join(workspace_dir, kb)

    if not os.path.exists(kb_dir):
        os.makedirs(kb_dir)

    kb_meta_path = os.path.join(kb_dir, 'kb.yml')

    kb_yaml = {}

    try:

        with open(kb_meta_path, 'r') as stream:
            try:
                kb_yaml = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                raise log.FatalMsg(exc)

            with open(os.path.join(kb_dir, 'kb.md'), 'r') as stream:
                kb_yaml['md']['text'] = utils.load_and_replace_media_with_b64(
                        kb_dir,
                        stream.read()
                    )

            

    except IOError:
        pass

    return kb_yaml

def get_kbs():

    kbs = {}

    for kb_yml_path in glob.glob('%s/*/kb.yml' % workspace_dir):

        current_name = kb_yml_path.split('/')[-2]
        
        kbs[current_name] = load_kb_meta(current_name)

    return kbs

def get_kb_by_uuid(kb_uuid):
    
    for kb_yml_path in glob.glob('%s/*/kb.yml' % workspace_dir):

        current_name = kb_yml_path.split('/')[-2]

        kb_data = load_kb_meta(current_name)

        if kb_data['uuid'] == kb_uuid:
            return kb_data

def delete_kb_by_uuid(kb_uuid):
        
    for kb_yml_path in glob.glob('%s/*/kb.yml' % workspace_dir):

        current_name = kb_yml_path.split('/')[-2]
        current_folder = os.path.dirname(kb_yml_path)

        kb_data = load_kb_meta(current_name)

        if kb_data['uuid'] == kb_uuid:
    
            tempfolder = tempfile.mkdtemp()

            log.warn("Moving %s to %s" % (current_folder, tempfolder))

            os.rename(current_folder, tempfolder)

            return kb_uuid

def update_kb_folder(old_kb_dir, new_kb_data):

    new_kb_dir = save_kb_meta(new_kb_data)

    if old_kb_dir != new_kb_dir:
        log.warn("Moved from %s to %s" % (utils.prettypath(old_kb_dir), utils.prettypath(new_kb_dir)))
        shutil.rmtree(old_kb_dir)


# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from sfsdk import log
import os
import os.path
import yaml
import shutil

settings = {}

default_technologies_path = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    "technologies.yml"
)

technologies = {}
technologies_path = None

def prepare_technologies(remote_tech_list):
    """ Merge current technologies with remote tech list """

    global technologies

    with open(default_technologies_path, 'r') as yaml_file:
        technologies = yaml.safe_load(yaml_file)

    for remote_tech in remote_tech_list:

        # Skip if empty or missing
        image_url = remote_tech.get('imageUrl')

        if not image_url or not '/' in image_url:
            log.debug("Skipping %s with no imageUrl" % (remote_tech['technology']))
            continue
        
        tag = remote_tech['imageUrl'].split('/')[-1]

        if not tag in technologies.keys():
            technologies[tag] = {}

        technologies[tag].update({
            'technology': remote_tech['technology'],
            'imageUrl': remote_tech['imageUrl']
        })

def load(base_dir):

    """ Load or create technologies settings file"""

    global technologies, technologies_path

    technologies_path = os.path.join(
        base_dir, 
        'technologies.yml'
    )

    try:
        with open(technologies_path, 'r') as yaml_file:
            technologies = yaml.safe_load(yaml_file)
    except OSError as e:
        os.makedirs(base_dir, exist_ok=True)
        shutil.copy(default_technologies_path, technologies_path)

        try:
            with open(technologies_path, 'r') as yaml_file:
                technologies = yaml.safe_load(yaml_file)
        except Exception as e:
            raise log.FatalMsg("Error loading default technology file")
    except yaml.YAMLError as e:
        raise log.FatalMsg("Settings file %s has broken YAML, delete it to recreate" % settings_path)

def save():
    """ Save settings file"""

    try:
        with open(technologies_path, 'w+') as outfile:
            yaml.dump(technologies, outfile, default_flow_style=False)
    except Exception as e:
        log.debug(str(e))
        raise log.FatalMsg("Error saving current settings" % technologies_path)




#!/usr/bin/env python3
# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest
from sfsdk import imgsettings, techsettings
from sfsdk import cli, containers, log
import tempfile
import os

base_dir = tempfile.mkdtemp(prefix='sfsdk-tests-')

# Needed here for the unittest.skip
containers.load()

class TestBuild(unittest.TestCase):

    def setUp(self):
        containers.load()
        imgsettings.load(base_dir)
        techsettings.load(base_dir)

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_build_python(self):
        cli.img_add(
                name = 'sdktestbuildpython', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestbuildpython',
                source= 'default',
                skip_base_img = False
                )

    @unittest.skipUnless(containers.image_exists('sf-java'),
                                 "Image sf-java is required")
    def test_build_java(self):
        cli.img_add(
                name = 'sdktestbuildjava', 
                from_img = 'sf-java', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestbuildjava',
                source= 'default',
                skip_base_img = False
                )

        
    @unittest.skipUnless(containers.image_exists('sf-golang'),
                                 "Image sf-golang is required")
    def test_build_golang(self):
        cli.img_add(
                name = 'sdktestbuildgolang', 
                from_img = 'sf-golang', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestbuildgolang',
                source= 'default',
                skip_base_img = False
                )
        
    @unittest.skipUnless(containers.image_exists('sf-ruby'),
                                 "Image sf-ruby is required")
    def test_build_ruby(self):
        cli.img_add(
                name = 'sdktestbuildruby', 
                from_img = 'sf-ruby', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestbuildruby',
                source= 'default',
                skip_base_img = False
                )

    @unittest.skipUnless(containers.image_exists('sf-php'),
                                 "Image sf-php is required")
    def test_build_php(self):
        cli.img_add(
                name = 'sdktestbuildphp', 
                from_img = 'sf-php', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestbuildphp',
                source= 'default',
                skip_base_img = False
                )

    @unittest.skipUnless(containers.image_exists('sf-dotnet'),
                                 "Image sf-dotnet is required")
    def test_build_dotnet(self):
        cli.img_add(
                name = 'sdktestbuilddotnet', 
                from_img = 'sf-dotnet', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestbuilddotnet',
                source= 'default',
                skip_base_img = False
                )

    @unittest.skipUnless(containers.image_exists('sf-solidity'),
                                 "Image sf-solidity is required")
    def test_build_solidity(self):
        cli.img_add(
                name = 'sdktestbuildsolidity', 
                from_img = 'sf-solidity', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestbuildsolidity',
                source= 'default',
                skip_base_img = False
                )

    def tearDown(self):

        for container in containers.dockerc.containers.list(all=True):
            if container.name.startswith('sdktest'):
                container.remove(force=True)

        for img in containers.dockerc.images.list():
            for tag in img.tags:
                if tag.startswith('sdktest'):
                    containers.dockerc.images.remove(tag, force=True)
        containers.dockerc.close()


if __name__ == '__main__':
    unittest.main()



# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from sfsdk import log
from sfsdk import exrsettings
from sfsdk import authsettings
from urllib.parse import urlparse
import boto3
import requests
import re
import time
import tempfile
import json

r = requests.Session()
#r.proxies.update({'https': 'http://127.0.0.1:8090'})
#r.verify = False

remote_archive_format = '%s-%s-%d.zip'

ctoken = None
url = None

hub_url = None
hub_token = None

def login_to_deployment(username, password, management_url):

    global r, ctoken, url

    url = management_url
    source = authsettings.current_source

    r.headers.update(
            authsettings.settings['sources'][source].get('headers', {})
        )

    r.post(
            url + '/handler', 
            json={
                "action":"doLogin",
                "username": username,
                "password": password
                }
            )
 
    res = r.post(
            url + '/management/sfadmin/handler', 
            json={"action":"getUserCToken"},
            allow_redirects=False
    )

    if res.status_code == 200:
        ctoken = res.json() ['ctoken']
        log.success("Logged in %s deployment as user %s" % (source, username))
    else:
        raise log.FatalMsg("Failed authentication on %s deployment as user %s, check credentials" % (source, username))

def get_s3_creds_via_deployment():

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":"getS3Credentials"} )

    return res.json()

def get_ecr_creds_via_deployment():

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":"getECRCredentials"} )

    return res.json()

def login_to_hub(username, password, management_hub_url):

    global hub_token, hub_url

    hub_url = management_hub_url
    
    source = authsettings.current_source
    r.headers.update(
            authsettings.settings['sources'][source].get('headers', {})
        )

    res = r.post(
            hub_url + '/rest/api/login-developer',
            json = {
                "username": username,
                "password": password
                } 
            )

    hub_repr = 'SecureFlag' if management_hub_url == authsettings.default_hub_url else source

    if res.status_code != 200:
        raise log.FatalMsg("Failed authentication to the %s hub as user '%s', check credentials in %s" % (hub_repr, username, authsettings.settings_path))
    else:
        log.success("Logged in the %s hub  as user '%s'" % (hub_repr, username))
        hub_token = res.json()

def get_s3_creds_via_hub():

    res = r.get(
            hub_url + '/rest/user/uploads/credentials',
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            }
        )

    return res.json()

def get_arn_via_hub():

    res = r.get(
            hub_url + '/rest/user/uploads/bucket',
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            }
        )

    return res.json()['arn']
    
def get_frameworks_via_hub():

    res = r.get(
            hub_url + '/rest/user/frameworks',
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            })
    
    if res.status_code == 204:
        return []

    return res.json()

def get_frameworks_via_deployment():

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":"getFrameworks", "ctoken": ctoken} )

    return res.json()

def get_technologies_via_hub():

    res = r.get(
            hub_url + '/rest/user/technologies',
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            }
        )
    
    if res.status_code == 204:
        return []

    return res.json()

def get_technologies_via_deployment():

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":"getHubTechnologyKBList", "ctoken": ctoken} )

    return res.json()

def get_technology_via_deployment_by_uuid(uuid):

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":" getHubTechnologyItem", "ctoken": ctoken, "uuid": uuid} )

    return res.json()

def get_technology_via_hub_by_uuid(uuid):

    res = r.get(
            hub_url + '/rest/user/technologies/%s' % uuid,
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            }
        )

    if res.status_code == 204:
        return {}

    return res.json()

def get_vulnerabilities_via_hub():

    res = r.get(
            hub_url + '/rest/user/vulnerabilities',
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            }
        )

    if res.status_code == 204:
        return []

    return res.json()

def get_vulnerabilities_via_deployment():

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":"getHubVulnerabilityKBList", "ctoken": ctoken} )
    
    return res.json()

def get_vulnerability_via_hub_by_uuid(uuid):

    res = r.get(
            hub_url + '/rest/user/vulnerabilities/%s' % uuid,
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            }
        )

    if res.status_code == 204:
        return {}

    return res.json()

def get_vulnerability_via_deployment_by_uuid(uuid):

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":"getHubKBItem", "ctoken": ctoken, "uuid": uuid } )

    return res.json()

def get_exercises_via_deployment():

    res = r.post(
            url + '/management/sfadmin/handler',
            json = {"action":"getExercises","ctoken": ctoken} )

    exercises_raw = res.json()

    exercises_raw = exercises_raw.get('exercises', [])

    return exercises_raw

def get_exercises_via_hub():

    res = r.get(
            hub_url + '/rest/user/exercises',
            headers = {
                'Authorization': 'Bearer %s' % (hub_token['value'])
            }
        )

    if res.status_code == 204:
        return {}

    return res.json()

def get_exercise_by_uuid(uuid):

    res = r.post(
        url + '/management/sfadmin/handler',
        json = {"action":"getExerciseDetails", "uuid": uuid, "ctoken": ctoken} )

    return res.json()

def push_exercise(exercise_json, exercises_uuids):

    if exercise_json['uuid'] in exercises_uuids:
        log.debug('Updating exercise with uuid %s' % exercise_json['uuid'])
        exercise_json['action'] = "updateExercise"
    else:
        log.debug('Adding exercise as new')
        exercise_json['action'] = "addExercise"

    exrsettings.fix_outgoing_exercise_data(exercise_json)

    exercise_json['ctoken'] = ctoken

    if log.verbose:
        with tempfile.NamedTemporaryFile(
                mode='w+',
                suffix='-push-exercise-last-pushed.json',
                delete=False
                ) as exercise_debug_file:
            log.debug('Saving JSON debug to %s' % (exercise_debug_file.name))
            json.dump(exercise_json, exercise_debug_file, indent=4)

    res = r.post(
            url + '/management/sfadmin/handler',
            json = exercise_json )

    return res.json()

def push_exercise_and_get_new_json(exercise_json, exercises_uuids):

    push_res = push_exercise(exercise_json, exercises_uuids)
    if push_res.get('result') != 'success':
        log.debug(str(push_res))
        raise log.FatalMsg("Exercise has not been pushed, its metadata might be corrupted of the exercise could be read-only")

    new_uuid = push_res['uuid']

    return get_exercise_by_uuid(new_uuid)

def push_to_s3(archive_path, aws_creds, username, aws_arn):

    stsclient = boto3.client(
        'sts',
        aws_access_key_id=aws_creds['accessKeyId'],
        aws_secret_access_key=aws_creds['secretAccessKey'],
        aws_session_token=aws_creds['sessionToken'],
    )
    account_identity = stsclient.get_caller_identity()

    s3client = boto3.client(
        's3',
        aws_access_key_id=aws_creds['accessKeyId'],
        aws_secret_access_key=aws_creds['secretAccessKey'],
        aws_session_token=aws_creds['sessionToken'],
    )

    remote_archive_name = remote_archive_format % (
        account_identity['Account'],
        username,
        int(time.time())
    )

    bucket = aws_arn.split(":")[5]

    try:
        res = s3client.upload_file(
            archive_path + '.zip', 
            bucket, 
            remote_archive_name
        )
    except boto3.exceptions.S3UploadFailedError as e:
        raise log.FatalMsg(str(e))

    log.success('Exercise has been published as %s' % (remote_archive_name))

def create_repo_and_get_image_url_via_deployment(image_name):

    res = r.post(
        url + '/management/sfadmin/handler',
        json = {"action":"createECRRepo", "ctoken": ctoken, "name": image_name} 
    )

    return res.json().get('url')


#!/usr/bin/env python3

# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import argparse
import json
import os
import pty
import random
import re
import shlex
import shutil
import subprocess
import sys
import tempfile
import time

import simplejson
from sfsdk import (authsettings, containers, exrsettings, frameworksettings,
                   imgsettings, kbsettings, log, remote, stacksettings,
                   techsettings, utils, web)

base_dir = os.path.expanduser('~/sf')

def img_snapshot(name, new_name, force):

    image_settings = imgsettings.settings.get('images', {}).get(name)

    if not new_name:
        new_name = imgsettings.get_snapshot_name(name)

    if not image_settings:
        raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add' to add it" % name)

    if new_name in imgsettings.settings.get('images', {}).keys():
        raise log.FatalMsg('Image %s already exists' % new_name)

    current_container_conf = imgsettings.get_container_conf(name)

    if containers.container_status(current_container_conf['name']) != 'running':
        raise log.FatalMsg("""Image is not running, consider running 'sfsdk img-watch %s' before running img-snapshot again.""" % (name))

    additional_paths = []

    paths_file = os.path.join(image_settings['build_dir'], '.sfsdk-watch.txt')

    additional_paths = []
    try:
        with open(paths_file, 'r') as stream:
            additional_paths = [ l for l in stream.read().split('\n') if l ]
    except OSError:
        log.debug("Error reading %s" % (paths_file))
        log.warn("Can't find result from img-watch, consider running 'sfsdk img-watch %s' before running img-snapshot again." % (name))

    monitored_paths = techsettings.technologies[image_settings['base_img']].get('default_paths', [])

    # Add additional paths without duplicates
    for additional_path in additional_paths:
        if not (additional_path in monitored_paths or additional_path + '/' in monitored_paths):
            monitored_paths.append(additional_path)

    container_settings = { v['name']:v['default'] for v in imgsettings.get_settings(new_name) }

    imgsettings.add_image_settings(
            new_name,
            base_img = image_settings['base_img'],
            build_dir = None,
            from_dir = image_settings['build_dir'],
            container_settings = container_settings
            )

    new_img_settings = imgsettings.settings.get('images', {}).get(new_name)
    new_build_dir = new_img_settings['build_dir']

    containers.save_container_snapshot(
            container_name = current_container_conf['name'],
            snapshot_folder = new_build_dir,
            from_image_name = image_settings['base_img'],
            modified_paths = monitored_paths
        )

    if monitored_paths:
        log.block("""The following paths have been exported from the running container and applied over the build folder.:

%s

The new project image has been saved as %s.
""" % (
            '\n'.join(monitored_paths),
            new_name
            )
        )

def img_techs(source):

    authsettings.set_source(source)

    source_type, management_url, username, password = authsettings.load_or_ask_for_creds()

    if source_type == 'deployment':
        remote.login_to_deployment(username, password, management_url)
        
        try:
            remote_technologies = remote.get_technologies_via_deployment()
        except simplejson.JSONDecodeError:
            raise log.FatalMsg("Error getting the technologies, is the source a deployment?")
    else:
        remote.login_to_hub(username, password, management_url)
        try:
            remote_technologies = remote.get_technologies_via_hub()
        except simplejson.JSONDecodeError:
            raise log.FatalMsg("Error getting the technologies via the Hub, are you a developer?")

    techsettings.prepare_technologies(remote_technologies)
    techsettings.save()

    log.success("Technologies has been updated:")
    log.block('\n'.join(techsettings.technologies), end='\n\n')

def img_add(name, from_img, build_dir, from_dir, app_dir):

    # Let's stick to the docker image name limitations
    if not re.match('[a-z0-9][a-z0-9_.-]*$', name):
        raise log.FatalMsg('Image name is not valid')

    if not from_img in techsettings.technologies.keys():
        raise log.FatalMsg(
            "Base technology does not exist in %s, run 'sfsdk img-techs' to update the list." % (techsettings.technologies_path)
        )

    container_settings = { v['name']:v['default'] for v in imgsettings.get_settings(name) }

    imgsettings.add_image_settings(
            name,
            base_img = from_img,
            build_dir = build_dir,
            from_dir = from_dir,
            app_dir = app_dir,
            container_settings = container_settings
            )

    image_settings = imgsettings.settings.get('images', {}).get(name)

    log.success("""New image %s has been added""" % name)
    log.block("""
    This new image is based on %s
    The build folder is %s
    """ % (
        from_img,
        utils.prettypath(image_settings['build_dir']),
        )
    )

def img_build(name, skip_base_img, source):

    image_settings = imgsettings.settings.get('images', {}).get(name)

    if not image_settings:
        raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add --from-img <base image> %s' to add it" % (name, name))

    base_img = image_settings['base_img']

    if not skip_base_img and not containers.image_exists(base_img):

        base_tech_settings = techsettings.technologies.get(base_img)
        if base_tech_settings == None:
            raise log.FatalMsg("Error, can't find technology image %s on %s technologies.yml" % (base_img, source))

        base_image_url = base_tech_settings.get('imageUrl')
        if not base_image_url:
            raise log.FatalMsg("Error, can't find technology image %s.imageUrl is set on %s technologies.yml" % (base_img, source))

        log.info('Pulling technology image %s from %s' % (base_img, base_image_url))

        pulled = True

        try:
            pull_stream = containers.pull_image(
                base_image_url
            )
        except log.FatalMsg as e:
            raise log.FatalMsg("Error pulling technology: %s" % e)

        for line in pull_stream:

            if 'error' in line:
                log.warn('Error pulling technology image: %s' % line['error'])
                pulled = False
            elif 'status' in line and line['status'] == 'Downloading':
                log.info('Pulling the image layer %s %s...%s' % (line['id'], line['progress'].split(' ')[-1], '\t\t\r'))

            sys.stdout.flush()

        containers.tag_image(base_image_url, base_img)

        if pulled:
            log.success("Technology image has been pulled and tagged as %s"  % (base_img))
        else:
            raise log.FatalMsg("Error pulling technology image, try manually with running 'docker pull %s' and 'docker tag %s %s'" % (base_image_url, base_image_url, base_img))

    log.info('Building exercise image %s...' % name)
    build_generator = containers.build_image(image_settings['build_dir'], name)

    built = False

    for line in build_generator:
        if 'stream' in line:
            output = line['stream'].strip('\n')
            log.block(output, end='\n', prefix='')

            if output.startswith('Successfully built'):
                built = True

    if built:
        log.success("Build complete")
    else:
        raise log.FatalMsg("The build of the image failed. Debug the build manually running\ndocker build -t %s %s" % (name, image_settings['build_dir']))

def img_shell(name, command):

    image_settings = imgsettings.settings.get('images', {}).get(name)

    if not image_settings:
        raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add' to add it" % name)

    container_settings = imgsettings.get_container_conf(name)
    container_name = container_settings['name']

    os.execvp("docker", [ "docker", "exec", "-it", container_name ] + shlex.split(command))

def img_run(name, force, exercise_name = None, flag_name = None):

    image_settings = imgsettings.settings.get('images', {}).get(name)

    if not image_settings:
        raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add' to add it" % name)

    container_settings = imgsettings.get_container_conf(name)
    container_name = container_settings['name']

    container_exists = containers.container_exists(container_name)
    container_status = containers.container_status(container_name)

    if container_exists and not force:
        if container_status == 'running':
            raise log.FatalMsg("Container %s is already running, you can stop and rebuild it." % container_name)
        else:
            raise log.FatalMsg("Container %s is not running, run 'sfsdk img-run %s --force' to delete and re-run" % (container_name, name))

    data = [
            [ "Container name:", container_name ],
            [ "RDP port:", '127.0.0.1:%s' % image_settings['container']['external_rdp_port'] ],
            [ "RDP credentials:", 'sf:' + image_settings['container']['rdp_password']  ],
            [ "Application port:", '127.0.0.1:%s' % image_settings['container']['external_http_port'] ]
        ]

    if exercise_name:
        container_settings['environment']['EXERCISE'] = exrsettings.get_exercise_environment_variable(exercise_name)
        data.append(
            [ "Exercise name:", exercise_name ]
        )
    elif flag_name:
        container_settings['environment']['EXERCISE'] = exrsettings.generate_exercise_environment(flag_name)
        data.append(
            [ "Flag name:", flag_name ]
        )
    else:
        data.append(
            [ "Exercise name:", 'none' ]
        )        

    if container_status != None:
        containers.container_remove(container_name)

    containers.run_instance(container_settings)

    log.success("Container %s has been started" % container_name)
        

    log.tablify(data)


def img_stop(names):

    image_names = imgsettings.resolve_images(names)

    for name in image_names:

        image_settings = imgsettings.settings.get('images', {}).get(name)

        if not image_settings:
            raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add' to add it" % name)

        container_settings = imgsettings.get_container_conf(name)
        container_name = container_settings['name']

        container_was_running = (containers.container_status(container_name) != 'running')

        if not container_was_running:

            if not containers.container_remove(container_name):
                log.warn("Container %s is not running" % container_name)
            else:
                log.success("Container %s has been stopped" % container_name)

def img_watch(name, folder, count):

    image_settings = imgsettings.settings.get('images', {}).get(name)

    if not image_settings:
        raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add' to add it" % name)

    output_file = os.path.join(image_settings['build_dir'], '.sfsdk-watch.txt')
    
    container_settings = imgsettings.get_container_conf(name)

    container_name = container_settings['name']
    log.debug("Writing img-watch output to %s" % output_file)
    log.success("Please wait, preparing %s to run img-watch.." % (container_name))

    containers.watch_container(container_name, watch_folder = folder)
    last_json_output = []

    try:

        i = 0
        found = 0

        while True:

            if count == 0:
                pass
            elif i > count:
                break
            else:
                i+=1

            exit_code, output = containers.exec_on_container(
                    container_name,
                    'cat /tmp/sfsdkwatch.json',
                    )

            if exit_code == 0 and output:

                found += 1
                if found == 1:
                    log.success("Started watching changes in %s folder.." % (folder))


                try:
                    json_output = json.loads(output)
                except json.JSONDecodeError as e:
                    continue

                if sorted(json_output) != sorted(last_json_output):
                    last_json_output = json_output[:]

                    log.success("%s Changed files and folders:" % (time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())))

                    directory_list = '\n'.join(last_json_output)
                    log.block(directory_list, end='\n\n')

                    with open(output_file, 'w+') as stream:
                        stream.write(directory_list)

            time.sleep(1)

    except KeyboardInterrupt:
        pass

def img_ls():

    data = [ ]

    for image_name, image_data in imgsettings.settings['images'].items():

        img_running = (containers.container_status(image_data['container']['container_name']) == 'running')

        data.append([
            image_name,
            'Yes' if img_running else 'No',
            image_data['container']['container_name'] if img_running else '',
            image_data['container']['external_rdp_port'] if img_running else '',
            'sf:' + image_data['container']['rdp_password'] if img_running else '',
            image_data['container']['external_http_port'] if img_running else '',
            ]
        )

    log.tablify(
            data,
            header = [
                'Name',
                'Running',
                'Container name',
                'RDP port',
                'RDP creds',
                'Application port'
            ]
        )

def img_rm(names):

    image_names = imgsettings.resolve_images(names)

    for name in image_names:

        image_settings = imgsettings.settings.get('images', {}).get(name)

        if not image_settings:
            raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add' to add it" % name)

        try:
            imgsettings.remove_image(name)
        except log.FatalMsg as e:
            log.warn(str(e))
        else:
            log.success("Image %s data has been removed" % name)


def exercise_edit(source):

    authsettings.set_source(source)

    source_type, management_url, username, password = authsettings.load_or_ask_for_creds()

    if source_type == 'deployment':
        remote.login_to_deployment(username, password, management_url)
        web.get_remote_technologies_func = remote.get_technologies_via_deployment
        web.get_remote_technology_func = remote.get_technology_via_deployment_by_uuid
        web.get_remote_vulnerabilities_func = remote.get_vulnerabilities_via_deployment
        web.get_remote_vulnerability_func = remote.get_vulnerability_via_deployment_by_uuid
        web.get_remote_frameworks_func = remote.get_frameworks_via_deployment
    else:
        remote.login_to_hub(username, password, management_url)
        web.get_remote_technologies_func = remote.get_technologies_via_hub
        web.get_remote_technology_func = remote.get_technology_via_hub_by_uuid
        web.get_remote_vulnerabilities_func = remote.get_vulnerabilities_via_hub
        web.get_remote_vulnerability_func = remote.get_vulnerability_via_hub_by_uuid
        web.get_remote_frameworks_func = remote.get_frameworks_via_hub

    web.run_editor()


def config(source):

    log.warn("""Warning: this saves your credentials in %s in an insecure way. Skip this (press Ctrl-C) to be prompted when authenticating.""" % authsettings.settings_path)

    authsettings.set_source(source)

    source_type, management_url, username, password = authsettings.load_or_ask_for_creds(
            doing_config=True
        )

    if not source in authsettings.settings['sources']:
        authsettings.settings['sources'][source] = {}

    authsettings.settings['sources'][source]['type'] = source_type
    authsettings.settings['sources'][source]['username'] = username
    authsettings.settings['sources'][source]['password'] = password

    if source_type == 'deployment':
        authsettings.settings['sources'][source]['url'] = management_url

    authsettings.save_to_file()

def publish(image_name, exercise_name, source):

    authsettings.set_source(source)

    source_type, management_url, username, password = authsettings.load_or_ask_for_creds()
    
    if source_type != 'hub':
        raise log.FatalMsg(
            "Error, a valid SecureFlag Exercise Hub account is required"
            )

    remote.login_to_hub(username, password, management_url)
    aws_creds = remote.get_s3_creds_via_hub()
    aws_arn = remote.get_arn_via_hub()
    get_remote_vulnerability_func = remote.get_vulnerability_via_hub_by_uuid
    get_remote_technology_func = remote.get_technology_via_hub_by_uuid

    tmp_dir_path = tempfile.mkdtemp(prefix='sfsdk-publish-archive-dir-')

    tmp_archive = tempfile.NamedTemporaryFile(
            prefix = 'sfsdk-publish-archive',
            )
    tmp_archive.close()

    log.debug('Building publish data in {tmp_dir_path} zipped to {tmp_archive.name}')

    shutil.copytree(
        imgsettings.settings['images'][image_name]['build_dir'], 
        os.path.join(tmp_dir_path, 'build')
    )

    exercise_data = exrsettings.get_exercise_by_name(exercise_name)
    with open(os.path.join(tmp_dir_path, 'exercise.json'), 'w') as f:
            json.dump(exercise_data, f, indent=4)

    for flag_i, flag_data in enumerate(exercise_data['flags']):
        kb_uuid = flag_data['kb']['uuid']

        kb_data = get_remote_vulnerability_func(kb_uuid)
        # Support both via_hub and via_deployment return messages
        if not kb_data or kb_data.get('errorMsg') == 'NotFound':
            kb_data = kbsettings.get_kb_by_uuid(kb_uuid)

        if not kb_data:
            raise log.FatalMsg(f"Error, can't find local or remote KB with uuid {kb_uuid}")

        with open(os.path.join(tmp_dir_path, f'kb_{flag_i}.json'), 'w') as f:
                json.dump(kb_data, f, indent=4)
    
    stack_uuid = exercise_data['stack']['uuid']
    stack_data = get_remote_technology_func(stack_uuid)
    if not stack_data or stack_data.get('errorMsg') == 'NotFound':
        # Support both via_hub and via_deployment return messages
        stack_data = stacksettings.get_stack_by_uuid(stack_uuid)

    if not stack_data:
        raise log.FatalMsg(f"Error, can't find local or remote stack with uuid {exercise_data['stack']['uuid']}")
    with open(os.path.join(tmp_dir_path, 'stack.json'), 'w') as f:
            json.dump(stack_data, f, indent=4)

    shutil.make_archive(tmp_archive.name, 'zip', tmp_dir_path)

    remote.push_to_s3(
            tmp_archive.name,
            aws_creds,
            username,
            aws_arn
    )

def deploy(image_name, exercise_name, source):

    authsettings.set_source(source)

    source_type, management_url, username, password = authsettings.load_or_ask_for_creds()

    if source_type != 'deployment':
        raise log.FatalMsg(
            "Error, deploying requires administrative privileges on a SecureFlag CE instance"
            )

    remote.login_to_deployment(username, password, management_url)
    aws_creds = remote.get_ecr_creds_via_deployment()

    exercise_settings = exrsettings.get_exercise_by_name(exercise_name)
    image_settings = imgsettings.settings.get('images', {}).get(image_name)

    if not image_settings:
        raise log.FatalMsg("Image %s does not exist, run 'sfsdk img-add' to add it" % image_name)

    if not containers.image_exists(image_name):
        raise log.FatalMsg("Image %s has not been built yet, run img-build to build it" % image_name)

    # Obtain the imageUrl from the instance
    image_url = remote.create_repo_and_get_image_url_via_deployment(image_name)

    # Extract the region name
    region_name = image_url.split('.')[3]

    # Save it to the exercise settings
    # exercise_settings['image']['imageUrl'] = image_url
    # exrsettings.save_exercise_meta(exercise_settings)

    exercises_uuids = [ e['uuid'] for e in remote.get_exercises_via_deployment() ]
    new_exercise_data = remote.push_exercise_and_get_new_json(exercise_settings, exercises_uuids)

    # Fix the folder based on the uuid
    exrsettings.update_exercise_folder(
            old_exercise_dir=os.path.join(exrsettings.workspace_dir, exercise_name),
            new_exercise_data=new_exercise_data
            )

    log.success("Exercise %s has been pushed with uuid %s" % (exercise_name, new_exercise_data['uuid']))

    # Push the image
    pushed = True
    push_stream = containers.tag_and_push_image(aws_creds, image_name, image_url, region_name)
    for line in push_stream:

        if 'error' in line:
            log.warn('Error pushing the image: %s' % line['error'])
            pushed = False
        elif 'status' in line and line['status'] == 'Pushing':
            log.info('Pushing the image layer %s %s...%s' % (line['id'], line['progress'].split(' ')[-1], '\t\t\r'))

        sys.stdout.flush()

    if pushed:
        log.success("Image %s has been pushed to %s" % (image_name, image_url))

def exercise_ls(source):

    authsettings.set_source(source)

    log.info('Local exercises list\n\n' + '\n'.join(exrsettings.ls_exercises()) + '\n')

def exercise_rm(names):

    for name in names:
        try:
            exrsettings.remove_exercise(name)
        except log.FatalMsg as e:
            log.warn(str(e))
        else:
            log.success("Exercise %s has been removed" % name)

def _get_argparser():

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest='subparser')

    imgtechs = subparsers.add_parser('img-techs', help='Update the list of base technologies')
    imgtechs.add_argument(
            '--source', dest='source', help='Source name')

    parser_add = subparsers.add_parser('img-add', help='Add new image')
    parser_add.add_argument(
        '-f', '--from-img', dest='from_img', help='Base technology image', required=True)
    parser_add.add_argument(
        '-b', '--build-dir', dest='build_dir', help='Build folder')
    parser_add.add_argument(
        '-i', '--import-build', dest='from_dir', help='Import build directory')
    parser_add.add_argument(
        '-a', '--import-app', dest='app_dir', help='Import application directory')
    parser_add.add_argument('name', help='New image name')

    parser_ls = subparsers.add_parser('img-ls', help='List images')

    parser_build = subparsers.add_parser('img-build', help='Build an image')
    parser_build.add_argument('-s', '--skip-base-img', dest='skip_base_img', help='Skip pulling base image', action='store_true')
    parser_build.add_argument('name', help='Image to build')
    parser_build.add_argument(
            '--source', dest='source', help='Source name')

    parser_run = subparsers.add_parser('img-run', help='Run last build')
    parser_run.add_argument('name', help='Project to run')
    parser_run.add_argument('-f', '--force', dest='force', help='Remove previous containers', action='store_true')
    parser_run.add_argument(
        '--exercise-name', '-en', dest='exercise_name', help='Pass exercise name'
        )
    parser_run.add_argument(
        '--flag-name', '-fn', dest='flag_name', help='Pass flag name'
        )
        
    parser_stop = subparsers.add_parser('img-stop', help='Stop running image')
    parser_stop.add_argument('names', nargs='+', help='Images names (or "all")')

    parser_shell = subparsers.add_parser('img-shell', help='Interact with a running image')
    parser_shell.add_argument('name', help='Run interactive shell on a running image')
    parser_shell.add_argument('-c', '--command', help='Command to execute', default='/bin/bash')

    parser_watch = subparsers.add_parser('img-watch', help='Keep track of the fs changes on a running image')
    parser_watch.add_argument(
        '-f', '--folder', dest='folder', help='Folder to watch', default='/home/sf')
    parser_watch.add_argument(
        '-c', '--count', dest='count', help=argparse.SUPPRESS, default=0, type=int)
    parser_watch.add_argument('name', help='Running image to watch')

    snapshot = subparsers.add_parser('img-snapshot', help='Save the current state as new')
    snapshot.add_argument(
        '--new-name', dest='new_name', help='The new image name')
    snapshot.add_argument(
        '-f', '--force', dest='force', help='Overwrite the export directory', action='store_true')
    snapshot.add_argument('name', help='Running image to snapshot')

    parser_rm = subparsers.add_parser('img-rm', help='Remove local image')
    parser_rm.add_argument('names', nargs='+', help='Images names')

    parser_exercise_ls = subparsers.add_parser('exercise-ls', help='List local exercises')
    parser_exercise_ls.add_argument(
            '--source', dest='source', help='Source name')

    exercise_edit = subparsers.add_parser('exercise-edit', help='Manage exercises via web editor')
    exercise_edit.add_argument(
            '--source', dest='source', help='Source name')

    exercise_rm = subparsers.add_parser('exercise-rm', help='Remove local exercise')
    exercise_rm.add_argument('names', nargs='+', help='Exercises names')

    push = subparsers.add_parser('deploy', help='Push exercise and image to a deployment')
    push.add_argument(
        '--image-name', dest='image_name', help='The image to push', required=True)
    push.add_argument(
        '--exercise-name', dest='exercise_name', help='The exercise name', required=True
        )
    push.add_argument(
            '--source', dest='source', help='Source name')

    publish = subparsers.add_parser('publish', help='Publish exercise and image on the SecureFlag Hub')
    publish.add_argument(
        '--image-name', dest='image_name', help='The image to publish', required=True)
    publish.add_argument(
        '--exercise-name', dest='exercise_name', help='The exercise name', required=True
        )
    publish.add_argument(
        '--source', dest='source', help='Source name')


    config = subparsers.add_parser('config', help='Configure authentication')
    config.add_argument(
            '--source', dest='source', help='Source name', default = 'default')

    parser.add_argument(
        '--verbose', '-v', dest='verbose', help='Verbose', action='store_true')

    return parser

def cli():

    parser = _get_argparser()

    kwargs = vars(parser.parse_args())

    if not kwargs.get('subparser'):
        parser.print_help()
        raise log.FatalMsg()

    imgsettings.load(base_dir)

    authsettings.load(base_dir)
    authsettings.set_source(kwargs.get('source'))

    # Initialize docker client
    containers.load()

    # Use another base_dir to support multiple sources
    sources_folder = os.path.join(
            base_dir,
            'srcs',
            authsettings.current_source
        )

    # Load settings from disk
    exrsettings.load(sources_folder)
    techsettings.load(sources_folder)
    kbsettings.load(sources_folder)
    stacksettings.load(sources_folder)
    frameworksettings.load(sources_folder)

    # Set verbose and remove from arguments
    if kwargs['verbose']:
        log.set_verbose()

    del kwargs['verbose']

    globals()[kwargs.pop('subparser').replace('-', '_')](**kwargs)



if __name__ == "__main__":

    try:
        cli()
    except log.FatalMsg as e:
        msg = str(e)
        if msg: log.warn(str(e))
        sys.exit(1)
    except KeyboardInterrupt as e:
        print('')



#!/usr/bin/env python3
# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest
from sfsdk import imgsettings, techsettings
from sfsdk import cli, containers, log
import yaml
import tempfile
import os

base_dir = tempfile.mkdtemp(prefix='sfsdk-tests-')

# Needed here for the unittest.skip
containers.load()

class TestRun(unittest.TestCase):

    def setUp(self):
        imgsettings.load(base_dir)
        techsettings.load(base_dir)

    @unittest.skipUnless(containers.image_exists('sf-python'),
                                 "Image sf-python is required")
    def test_run_new_no_force(self):
        cli.img_add(
                name = 'sdktestrunpython', 
                from_img = 'sf-python', 
                build_dir = None, 
                from_dir = None,
                app_dir = None)

        cli.img_build(
                name = 'sdktestrunpython',
                source= 'default',
                skip_base_img = False
                )

        try:
            cli.img_run(
                    name = 'sdktestrunpython',
                    force = False
                    )
        except log.FatalMsg:
            self.fail("Unexpected Exception")


# TODO: fix this
    # @unittest.skipUnless(containers.image_exists('sf-python'),
    #                              "Image sf-python is required")
    # def test_run_with_environment(self):
    #     cli.img_add(
    #             name = 'sdktestrunpython2', 
    #             from_img = 'sf-python', 
    #             build_dir = None, 
    #             from_dir = None,
    #             app_dir = None)

    #     settings = {}
    #     with open(os.path.join(base_dir, 'images.yml'), 'r') as yaml_file:
    #         settings = yaml.safe_load(yaml_file)

    #     settings['images']['sdktestrunpython2']['container']['environment']['SSH_KEY'] = 'bogus'

    #     with open(os.path.join(base_dir, 'images.yml'), 'w') as yaml_file:
    #         yaml.dump(settings, yaml_file)

    #     cli.img_build(
    #             name = 'sdktestrunpython2',
    #             source= 'default',
    #             skip_base_img = False
    #             )

    #     try:
    #         cli.img_run(
    #                 name = 'sdktestrunpython2',
    #                 force = False
    #                 )
    #     except log.FatalMsg:
    #         self.fail("Unexpected Exception")

    #     _, output = cli.containers.exec_on_container(
    #             'sdktestrunpython2-inst',  
    #             """sh -c "$SSH_KEY" """)
    #     self.assertEqual(output, "bogus")

    def tearDown(self):

        for container in containers.dockerc.containers.list(all=True):
            if container.name.startswith('sdktest'):
                container.remove(force=True)

        for img in containers.dockerc.images.list():
            for tag in img.tags:
                if tag.startswith('sdktest'):
                    containers.dockerc.images.remove(tag, force=True)

        containers.dockerc.close()

if __name__ == '__main__':
    unittest.main()



# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import setuptools
import os
import shutil

with open("README.md", "r") as fh:
    long_description = fh.read()

bins_folder='build/_bins'
os.makedirs(bins_folder, exist_ok=True)
shutil.copyfile('sfsdk/cli.py', bins_folder+'/sfsdk')

setuptools.setup(
    name="sfsdk",
    author="SecureFlag",
    author_email="info@secureflag.com",
    description="SDK for the SecureFlag training platform",
    long_description=long_description,
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    long_description_content_type="text/markdown",
    url="https://gitlab.com/secureflag-community/sdk",
    packages=['sfsdk'],
    install_requires=[
        'ruamel.yaml',
        'Flask',
        'jinja2',
        'PyYAML',
        'docker',
        'boto3',
        'prettytable',
        'simplejson'
        ],
    scripts=[
        bins_folder + '/sfsdk',
        ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    ],
    include_package_data=True
)



#!/usr/bin/env python3
# This file is part of the SecureFlag Platform.
# Copyright (c) 2020 SecureFlag Limited.

# This program is free software: you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by  
# the Free Software Foundation, version 3.

# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.

# You should have received a copy of the GNU General Public License 
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest
from sfsdk import cli, containers, log
from sfsdk import imgsettings, techsettings, authsettings, exrsettings, stacksettings, kbsettings, frameworksettings
import json
import requests
import tempfile
import multiprocessing
import time
import os

base_dir = tempfile.mkdtemp(prefix='sfsdk-tests-') 
settings_path = os.path.join(
        base_dir,
        'login.yml' 
    )

# Load authsettings from the real folder
auth_base_dir = os.path.expanduser('~/sf')
authsettings.load(auth_base_dir)
exrsettings.load(base_dir)
techsettings.load(base_dir)

#log.set_verbose()

developer_source = 'developer-test-account'

class TestEdit(unittest.TestCase):

    def setUp(self):
        techsettings.load(base_dir)
        exrsettings.load(base_dir)

        self.p = multiprocessing.Process(target=cli.exercise_edit, args=(developer_source,))
        self.p.start()

        # Give some time for flask to spin up
        time.sleep(2)

        res = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getUserCToken"},
        ).json()
        self.ctoken = res.get('ctoken')
        self.assertTrue(self.ctoken)

    def tearDown(self):
        self.p.terminate()
        self.p.join()


    @unittest.skipUnless(authsettings.settings.get('sources', {}).get(developer_source),
                                 "No developer creds in login.yml")
    def test_wrong_ctoken(self):

        res = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getAllKbs", "ctoken": '1'*19},
        ).json()
        self.assertEqual(res.get('result'), 'error')

    @unittest.skipUnless(authsettings.settings.get('sources', {}).get(developer_source),
                                 "No developer creds in login.yml")
    def test_exercise_api(self):

        # Check exercise list is empty
        res_exercises = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getExercises", "ctoken": self.ctoken },
        ).json()
        self.assertEqual(len(res_exercises), 0)

        # Push exercise sample
        test_folder = os.path.dirname(os.path.abspath(__file__))
        with open(os.path.join(test_folder, 'exercise-sample.json'), "r") as f:
            json_data = json.load(f)

        json_data['action'] = 'addExercise'
        json_data['ctoken'] = self.ctoken

        res_add_exercise = requests.post(
                'http://localhost:5000/handler', 
                json=json_data,
        ).json()
        self.assertEqual(res_add_exercise.get('result'), 'success')

        # Check the list of exercises has now 1 length
        res_exercises = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getExercises", "ctoken": self.ctoken },
        ).json()
        self.assertEqual(len(res_exercises), 1)

        # Check exercise detail
        res_exercise_details = requests.post(
                'http://localhost:5000/handler', 
                json={
                    "action":"getExerciseDetails", 
                    "ctoken": self.ctoken, 
                    "uuid": res_add_exercise.get("uuid") 
                },
        ).json()
        self.assertEqual(res_exercise_details['title'], json_data['title'])

        # Update exercise detail
        json_data['action'] = 'updateExercise'
        json_data['title'] = 'NEW TITLE'
        json_data['uuid'] = res_add_exercise.get("uuid")

        res_add_exercise = requests.post(
                'http://localhost:5000/handler', 
                json=json_data,
        ).json()
        self.assertEqual(res_add_exercise.get('result'), 'success')

        # Check the list of exercises is still 1 length
        res_exercises = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getExercises", "ctoken": self.ctoken },
        ).json()
        self.assertEqual(len(res_exercises), 1)

        # Check exercise detail
        res_exercise_details = requests.post(
                'http://localhost:5000/handler', 
                json={
                    "action":"getExerciseDetails", 
                    "ctoken": self.ctoken, 
                    "uuid": res_add_exercise.get("uuid") 
                },
        ).json()
        self.assertEqual(res_exercise_details['title'], json_data['title'])

    @unittest.skipUnless(authsettings.settings.get('sources', {}).get(developer_source),
                                 "No developer creds in login.yml")
    def test_kb_api(self):

        res_kbs = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getAllKbs", "ctoken": self.ctoken },
        ).json()
        self.assertTrue(res_kbs)
        self.assertTrue(res_kbs[0].get('uuid'))

        res_kb = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getKBItem", "ctoken": self.ctoken, "uuid": res_kbs[0].get('uuid') },
        ).json()
        self.assertTrue(res_kb)
        self.assertTrue(res_kb.get('uuid'))

    @unittest.skipUnless(authsettings.settings.get('sources', {}).get(developer_source),
                                 "No developer creds in login.yml")
    def test_stack_api(self):
        res_stacks = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getAllStacks", "ctoken": self.ctoken },
        ).json()
        self.assertTrue(res_stacks)
        self.assertTrue(res_stacks[0].get('uuid'))

        res_stack = requests.post(
                'http://localhost:5000/handler', 
                json={"action":"getStackItem", "ctoken": self.ctoken, "uuid": res_stacks[0].get('uuid') },
        ).json()
        self.assertTrue(res_stack)
        self.assertTrue(res_stack.get('uuid'))

if __name__ == '__main__':
    unittest.main()
